!delfile "Check Sections.nsi"

#==========================================
# COMPRESSION SETTINGS
#==========================================
    CRCCheck force
    SetCompressor /SOLID /FINAL lzma



#==========================================
# DEFINES
#==========================================
    !include "Defines.nsi"
    !define NAME "Force System File Update"
    !define WM_CLOSE 0x0010
    !define WM_COMMAND 0x0111
    !define FORCESYSTEMFILEUPDATE



#==========================================
# INCLUDES
#==========================================
    !include "Sections.nsh"
    !addplugindir "Assets\Plugins"



#==========================================
# INSTALLER INFORMATION
#==========================================
    Name "${NAME}"
    OutFile "Assets\Tools\${NAME}.exe"



#==========================================
# INTERFACE SETTINGS
#==========================================
    Caption "${NAME}"
    SubCaption 1 " "
    BrandingText "${COPYRIGHT} ${PRODUCT_PUBLISHER}"
    XPStyle on
    ChangeUI IDD_INST "Assets\Dialogs\${NAME}.exe"
    InstallColors /windows
    ShowInstDetails show
    InstallButtonText "Update"
    Icon "Assets\Images\Icons\Force System File Update.ico"
    SpaceTexts "none"

    Var AppDir
    Var FileSize
    Var Office
    Var UpdatedFileCount



#==========================================
# PAGES
#==========================================
      ComponentText "Check the files you would like to force update. When force updating a file, the current modified file is not backed up." "" "Select system files to update:"
      Page components "" "" "CheckForSelections2"
      Page instfiles



#==========================================
# INSTALLER MACROS
#==========================================
    !macro ForceUpdateFile UNUSED FileName TargetFolder
      !define UNIQUE_LINE ${__LINE__}

      #CHECK IF FILE EXISTS
      IfFileExists "${TargetFolder}\${FileName}" 0 END_${UNIQUE_LINE}
      IfFileExists "${INST_RESOURCES}\${FileName}" 0 END_${UNIQUE_LINE}

      #READ REGISTRY FILE SIZE
      ReadRegStr $0 HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}"
      StrCmp $0 "" END_${UNIQUE_LINE}

      #DISPLAY CURRENTLY INSTALLING FILE
      !insertmacro SetDetailsPrint "Force Updating File: ${FileName}"

      #MODIFY FILE
      CopyFiles /SILENT "${TargetFolder}\${FileName}" "${INST_TEMPFILES}"
      StrCpy "$OUTDIR" "${INST}"
      ExecWait '"${INST_TOOLS}\ResHacker.exe" -script "${INST_RESOURCES}\${FileName}\${FileName}.txt"'
      IfFileExists "${INST_NEWFILES}\${FileName}" +3
      DeleteRegValue HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}"
      Goto END_${UNIQUE_LINE}

      #WRITE NEW FILE SIZE TO REGISTRY
      Push "${INST_NEWFILES}\${FileName}"
      Call GetFileSize
      Pop $FileSize
      WriteRegStr HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}" "$FileSize"

      #REPLACE FILE IN CACHE
      IfFileExists "$SYSDIR\dllcache\${FileName}" 0 +2
      CopyFiles /SILENT "${INST_NEWFILES}\${FileName}" "$SYSDIR\dllcache\${FileName}"
      IfFileExists "$WINDIR\ServicePackFiles\i386\${FileName}" 0 +2
      CopyFiles /SILENT "${INST_NEWFILES}\${FileName}" "$WINDIR\ServicePackFiles\i386\${FileName}"

      #REPLACE FILE IN TARGET FOLDER
      Delete "${TargetFolder}\${FileName}.${PRODUCT_NAME}"
      Rename "${INST_NEWFILES}\${FileName}" "${TargetFolder}\${FileName}.${PRODUCT_NAME}"
      ExecCmd::exec '"${INST_TOOLS}\MoveEx.exe" "${TargetFolder}\${FileName}.${PRODUCT_NAME}" "${TargetFolder}\${FileName}"'

      #COUNT UPDATED FILES
      IntOp $UpdatedFileCount $UpdatedFileCount + 1

      SetRebootFlag true

      END_${UNIQUE_LINE}:
      !undef UNIQUE_LINE

      #CREATE "CHECK SECTIONS.nsi" FILE
      !appendfile "Check Sections.nsi" 'StrCmp "$0" "${FileName}" 0 +6$\n'
      !appendfile "Check Sections.nsi" '!insertmacro SelectSection "${${FileName}}"$\n'
    !macroend


    !macro ComCtl32 UNUSED FileName TargetFolder Number
      !define UNIQUE_LINE ${__LINE__}

      #CHECK IF FILE EXISTS
      IfFileExists "${TargetFolder}\${FileName}" 0 END_${UNIQUE_LINE}
      IfFileExists "${INST_RESOURCES}\${FileName}" 0 END_${UNIQUE_LINE}

      #READ REGISTRY FILE SIZE
      ReadRegStr $0 HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}${Number}"
      StrCmp $0 "" END_${UNIQUE_LINE}

      #DISPLAY CURRENTLY INSTALLING FILE
      !insertmacro SetDetailsPrint "Force Updating File: ${FileName}"

      #MODIFY FILE
      SetRebootFlag true
      CopyFiles /SILENT "${TargetFolder}\${FileName}" "${INST_TEMPFILES}"
      StrCpy "$OUTDIR" "${INST}"
      ExecWait '"${INST_TOOLS}\ResHacker.exe" -script "${INST_RESOURCES}\${FileName}\${FileName}.txt"'
      IfFileExists "${INST_NEWFILES}\${FileName}" +3
      DeleteRegValue HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}${Number}"
      Goto END_${UNIQUE_LINE}
      Rename "${INST_NEWFILES}\${FileName}" "${INST_NEWFILES}\${FileName}${Number}"

      #WRITE FILE SIZE TO REGISTRY
      Push "${INST_NEWFILES}\${FileName}${Number}"
      Call GetFileSize
      Pop $FileSize
      WriteRegStr HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}${Number}" "$FileSize"

      #REPLACE FILE IN TARGET FOLDER
      Delete "${TargetFolder}\${FileName}.${PRODUCT_NAME}"
      Delete "${TargetFolder}\${FileName}.delete"
      Rename "${TargetFolder}\${FileName}" "${TargetFolder}\${FileName}.delete"
      Rename "${INST_NEWFILES}\${FileName}${Number}" "${TargetFolder}\${FileName}"

      #COUNT UPDATED FILES
      IntOp $UpdatedFileCount $UpdatedFileCount + 1

      END_${UNIQUE_LINE}:
      !undef UNIQUE_LINE

      #CREATE "CHECK SECTIONS.nsi" FILE
      !appendfile "Check Sections.nsi" 'StrCmp "$0" "${FileName}" 0 +6$\n'
      !appendfile "Check Sections.nsi" '!insertmacro SelectSection "${${FileName}}"$\n'
    !macroend


    !macro OfficeFiles MacroName FileName SubKey Entry
      #GET FILE PATH
      ReadRegStr $Office HKLM "${SubKey}" "${Entry}"
      Push $Office
      Call GetParent
      Pop $Office

      !insertmacro ${MacroName} "System Files\Office" "${FileName}" "$Office"
      IfFileExists "$Office\${FileName}.${PRODUCT_NAME}" 0 +9
      Delete "$Office\${FileName}.delete"
      Rename "$Office\${FileName}" "$Office\${FileName}.delete"
      Rename "$Office\${FileName}.${PRODUCT_NAME}" "$Office\${FileName}"
      Delete "$Office\${FileName}.delete"

      Push "$Office\${FileName}"
      Call GetFileSize
      Pop $FileSize
      WriteRegStr HKLM "Software\${PRODUCT_NAME}\FILES" "${FileName}" "$FileSize"
    !macroend


    !macro SetDetailsPrint TextToPrint
      SetDetailsPrint both
      DetailPrint "${TextToPrint}"
      SetDetailsPrint none
    !macroend


    !appendfile "Check Sections.nsi" 'StrCmp "$0" "SYSTEMFILES" 0 +6$\n'
    !appendfile "Check Sections.nsi" '!insertmacro SelectSection "${SYSTEMFILES}"$\n'



#==========================================
# SECTION - PRE UPDATE (HIDDEN)
#==========================================
    Section "-Pre Update"
      !insertmacro SetDetailsPrint "Preparing Force Update..."
      !insertmacro SetDetailsPrint ""

      #CREATE DIRECTORIES
      IfFileExists "${INST}" 0 +3
      CreateDirectory "${INST_NEWFILES}"
      CreateDirectory "${INST_TEMPFILES}"

      ExecCmd::exec "${INST_TOOLS}\wfpdisable.exe"
    SectionEnd



#==========================================
# SECTION - SYSTEM FILES
#==========================================
    !include "System Files.nsi"
    !insertmacro SystemFiles "/o" "" "ForceUpdateFile"



#==========================================
# SECTION - MISC FILES
#==========================================
    Section /o "Logon Screen" LOGONUI.EXE
        !insertmacro ForceUpdateFile "Extras\Logon Screen" "logonui.exe" "$SYSDIR"
    SectionEnd

    Section /o "Boot Screen" NTOSKRNL.EXE
        !insertmacro ForceUpdateFile "Extras\Boot Screen\Normal" "ntkrnlmp.exe" "$SYSDIR"
        !insertmacro ForceUpdateFile "Extras\Boot Screen\Normal" "ntkrnlpa.exe" "$SYSDIR"
        !insertmacro ForceUpdateFile "Extras\Boot Screen\Normal" "ntoskrnl.exe" "$SYSDIR"

        IfFileExists "$SYSDIR\ntkrnlpa.exe.${PRODUCT_NAME}" 0 +2
      	ExecCmd::exec  "${INST_TOOLS}\XVI32.exe $SYSDIR\ntkrnlpa.exe.${PRODUCT_NAME} /S=${INST_TOOLS}\ReplacePalette.xsc"

        IfFileExists "$SYSDIR\ntoskrnl.exe.${PRODUCT_NAME}" 0 +2
      	ExecCmd::exec  "${INST_TOOLS}\XVI32.exe $SYSDIR\ntoskrnl.exe.${PRODUCT_NAME} /S=${INST_TOOLS}\ReplacePalette.xsc"

        IfFileExists "$SYSDIR\ntkrnlmp.exe.${PRODUCT_NAME}" 0 +2
      	ExecCmd::exec  "${INST_TOOLS}\XVI32.exe $SYSDIR\ntkrnlmp.exe.${PRODUCT_NAME} /S=${INST_TOOLS}\ReplacePalette.xsc"
    SectionEnd



#==========================================
# SECTION - POST UPDATE (HIDDEN)
#==========================================
    Section "-Cleanup"
      Delete "${INST_TOOLS}\ResHacker.ini"
      RMdir "${INST_BACKUP}"
      RMdir "${INST_LOGS}"
      RMdir "${INST_NEWFILES}"
      RMdir "${INST_RESOURCES}"
      RMdir /r "${INST_TEMPFILES}"
    SectionEnd



#==========================================
# SECTION - POST UPDATE (HIDDEN)
#==========================================
    Section "-Post Update"
      StrCmp "$UpdatedFileCount" "0" 0 MORETHANZERO
      !insertmacro SetDetailsPrint "The selected files could not be updated."
      Goto END

      MORETHANZERO:
      StrCmp "$UpdatedFileCount" "1" 0 MORETHANONE
      !insertmacro SetDetailsPrint ""
      !insertmacro SetDetailsPrint "$UpdatedFileCount file has been updated and will be applied on the next reboot."
      Goto +7

      MORETHANONE:
      !insertmacro SetDetailsPrint ""
      !insertmacro SetDetailsPrint "$UpdatedFileCount files have been updated and will be applied on the next reboot."

      Call DeleteIconCache
      WriteRegDWORD HKLM "SYSTEM\CurrentControlSet\Control\Session Manager" "AllowProtectedRenames" 0x00000001

      END:
      SetOutPath $TEMP
      System::Free 0
    SectionEnd



#==========================================
# INSTALLER FUNCTIONS
#==========================================
    !include "Functions.nsi"

    Function .onInit
      InitPluginsDir

      Call OneInstance
      System::Call "kernel32::GetModuleHandle(t 'shell32.dll') i .s"
      System::Call "kernel32::GetProcAddress(i s, i 680) i .r0"
      System::Call "::$0() i .r0"
      StrCmp "$0" "1" +3
      MessageBox MB_OK "The ${NAME} tool cannot be run on a limited user account. $\nThe ${NAME} will now quit." /SD IDOK
      Quit

      StrCpy $UpdatedFileCount "0"
      SetShellVarContext all

      #FORCE FILE UPDATE
      Push $CMDLINE
      Call GetNextParm
      Pop $0
      StrCpy $9 ""

      PARMSLOOP:
      Call GetNextParm
      Pop $0
      StrCmp $0 "" PARMSDONE
      StrCpy $1 $0 2
      StrCmp $1 "_=" PARMSDONE
      StrCmp $0 "/S" PARMSLOOP
      StrCpy $0 $0 "" 1
      !include "Check Sections.nsi"
      Goto PARMSLOOP

      PARMSDONE:
      Pop $0
    FunctionEnd


    Function .onGuiEnd
      SetOutPath $TEMP
      System::Free 0
    FunctionEnd


    Function "CheckForSelections2"
      !insertmacro SectionFlagIsSet ${SYSTEMFILES} ${SF_SELECTED} END ""
      !insertmacro SectionFlagIsSet ${SYSTEMFILES} ${SF_PSELECTED} END ""
      !insertmacro SectionFlagIsSet ${NTOSKRNL.EXE} ${SF_SELECTED} END ""
      !insertmacro SectionFlagIsSet ${LOGONUI.EXE} ${SF_SELECTED} END ""
      MessageBox MB_OK|MB_ICONEXCLAMATION "You did not select any items to update.  Please look through the options and select only the items that you want to force update."
      Abort
      END:
    FunctionEnd


    Function GetNextParm
      Exch $0
      Push $1
      Push $9
      Push $8
      StrCpy $1 ""
      TRIMLEADING:
        StrCpy $9 $0 1
        StrCmp $9 "" Done
        StrCmp $9 '"' DELIMITQUOTE
        StrCmp $9 " " "" DELIMITSPACE
        StrCpy $0 $0 "" 1
        Goto TRIMLEADING
      DELIMITQUOTE:
        StrCpy $0 $0 "" 1
        Goto COPYPARM
      DELIMITSPACE:
        StrCpy $9 " "
      COPYPARM:
        StrCpy $8 $0 1
        StrCmp $8 "" DONE
        StrCpy $0 $0 "" 1
        StrCmp $8 $9 DONE
        StrCpy $1 $1$8
        Goto COPYPARM
      DONE:
        Pop $8
        Pop $9
        Exch $1
        Exch
        Exch $0
        Exch
    FunctionEnd